from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
import copy


DEFAULT_CONFIG = {
    "width": "100%",
    "height": 720,
    "cache": {"enable": False},
    "mode": "sv",
    "debugger": "false",
    "icon": "ant",
    "outline": "",
    "counter": {
        "enable": True,
    },
    "lang": "zh_CN",
    "toolbar": [
        "emoji",
        "headings",
        "bold",
        "italic",
        "strike",
        "link",
        "|",
        "list",
        "ordered-list",
        "check",
        "outdent",
        "indent",
        "|",
        "quote",
        "line",
        "code",
        "inline-code",
        "insert-after",
        "table",
        "|",
        "upload",
        "fullscreen",
        "export",
        "|",
        "outline",
    ],
    "upload": {
        "storage": "local",
        "url": "/upload/",  # 上传接口地址
        "max": 5 * 1024 * 1024,  # 5MB
        "accept": "image/png,image/jpeg,image/gif,image/webp",  # 允许类型
        "fieldName": "file[]",
        "multiple": True,
    },
}


def deep_update(d, u):
    for k, v in u.items():
        if isinstance(v, dict) and k in d and isinstance(d[k], dict):
            d[k] = deep_update(d[k], v)
        else:
            d[k] = v
    return d


class VditorConfig(dict):
    def __init__(self, config_name="default"):
        # 总是从深度复制的默认配置开始
        config = copy.deepcopy(DEFAULT_CONFIG)

        # 获取 settings.py 中的 VDITOR_CONFIGS
        custom_configs = getattr(settings, "VDITOR_CONFIGS", None)

        if custom_configs:
            if not isinstance(custom_configs, dict):
                raise ImproperlyConfigured(
                    "VDITOR_CONFIGS setting must be a dictionary type."
                )

            if config_name == "default":
                # 如果是默认配置，直接将 VDITOR_CONFIGS 视为覆盖项
                config = deep_update(config, custom_configs)
            elif config_name in custom_configs:
                # 如果是具名配置，从 custom_configs 中查找
                named_config = custom_configs[config_name]
                if not isinstance(named_config, dict):
                    raise ImproperlyConfigured(
                        'VDITOR_CONFIGS["%s"] setting must be a dictionary type.'
                        % config_name
                    )
                config = deep_update(config, named_config)
            # 如果具名配置在 custom_configs 中不存在，则静默使用默认配置

        self.update(config)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.files.storage import FileSystemStorage
from .configs import VditorConfig


def upload_to_storage(file_obj, pathname=None):
    upload_config = VditorConfig("default").get("upload", {})

    storage_type = upload_config.get("storage", "local")
    if storage_type == "oss":
        import oss2

        return upload_to_oss(file_obj, pathname, oss2)
    elif storage_type == "tos":
        import tos

        return upload_to_tos(file_obj, pathname, tos)
    elif storage_type == "local":
        return upload_to_local(file_obj, pathname)
    else:
        raise ImproperlyConfigured(f"不支持: {storage_type}")


def _get_remote_filename(file_name, pathname):
    local_path = VditorConfig("default").get("upload", {}).get("local_path", "")
    parts = [part for part in [local_path, pathname, file_name] if part]  # 过滤掉空值
    return "/".join(parts)


def upload_to_local(file_obj, pathname=None):
    filename_with_path = _get_remote_filename(file_obj.name, pathname)

    # 使用 Django 默认的 FileSystemStorage，它会自动使用 settings.MEDIA_ROOT
    fs = FileSystemStorage()

    # 保存文件，fs.save 会处理重名并返回实际保存的（可能重命名后的）路径
    actual_filename = fs.save(filename_with_path, file_obj)

    # 使用 fs.url 来获取正确的、包含所有路径的 URL
    return fs.url(actual_filename)


def upload_to_oss(file_obj, pathname, oss2):
    oss_config = settings.DJ_IMAGE_UPLOADER_OSS_CONFIG
    filename = _get_remote_filename(file_obj.name, pathname)
    try:
        auth = oss2.Auth(oss_config["ACCESS_KEY_ID"], oss_config["ACCESS_KEY_SECRET"])
        bucket = oss2.Bucket(auth, oss_config["ENDPOINT"], oss_config["BUCKET_NAME"])
        bucket.put_object(filename, file_obj)
        return (
            f"https://{oss_config['BUCKET_NAME']}.{oss_config['ENDPOINT']}/{filename}"
        )
    except KeyError as e:
        raise ImproperlyConfigured(f"缺少OSS配置: {e}")


def upload_to_tos(file_obj, pathname, tos):
    tos_config = settings.DJ_IMAGE_UPLOADER_TOS_CONFIG
    filename = _get_remote_filename(file_obj.name, pathname)
    try:
        client = tos.TosClientV2(
            region=tos_config["REGION"],
            ak=tos_config["ACCESS_KEY"],
            sk=tos_config["SECRET_KEY"],
            endpoint=tos_config["ENDPOINT"],
        )
        client.put_object(tos_config["BUCKET_NAME"], filename, content=file_obj)
        return (
            f"https://{tos_config['BUCKET_NAME']}.{tos_config['ENDPOINT']}/{filename}"
        )
    except KeyError as e:
        raise ImproperlyConfigured(f"缺少TOS配置: {e}")



from django.http import JsonResponse
from .configs import VditorConfig
from django.views.decorators.http import require_POST
from django.contrib.auth.decorators import login_required
from .storage import upload_to_storage
import uuid
from django.views.decorators.csrf import csrf_exempt


VDITOR_CONFIGS = VditorConfig("default")


@login_required
@require_POST
@csrf_exempt
def vditor_images_upload_view(request):
    """处理图片上传请求"""
    file_list = request.FILES.getlist("file[]")

    # 验证文件存在
    if not file_list:
        return JsonResponse(
            {
                "code": 400,
                "msg": "未收到上传文件",
                "data": {"errFiles": [], "succMap": {}},
            },
            status=400,
        )

    succ_map = {}
    err_files = []

    allowed_types = VDITOR_CONFIGS["upload"]["accept"].split(",")
    max_size = VDITOR_CONFIGS["upload"]["max"]
    user_id = request.user.id

    # 初始化pathname变量
    if isinstance(user_id, uuid.UUID):
        pathname = f"user-{user_id}"
    else:
        # 将数字 ID 格式化为 8 位带前导零
        try:
            pathname = f"user-{int(user_id):08d}"
        except ValueError:
            # 如果既不是 UUID 也不是数字，保留原始值
            pathname = f"user-{str(user_id)}"

    for file_obj in file_list:

        # 文件类型验证
        if file_obj.content_type not in allowed_types:
            err_files.append(file_obj.name)
            continue

        # 文件大小验证
        if file_obj.size > max_size:
            err_files.append(file_obj.name)
            return JsonResponse(
                {
                    "code": 413,
                    "msg": f"文件大小超过{max_size//1024//1024}MB限制",
                    "data": {"errFiles": [file_obj.name], "succMap": {}},
                },
                status=413,
            )

        try:
            # 上传文件
            file_url = upload_to_storage(file_obj, pathname)
            succ_map[file_obj.name] = file_url
        except Exception as e:
            err_files.append(file_obj.name)

    return JsonResponse(
        {
            "code": 0 if len(err_files) == 0 else 500,
            "msg": f"成功上传 {len(succ_map)} 个文件",
            "data": {
                "errFiles": [],
                "succMap": succ_map,
            },
        }
    )



from django.db import models
from django import forms
from .widgets import VditorWidget


class VditorTextField(models.TextField):
    def __init__(self, *args, **kwargs):
        self.config_name = kwargs.pop("config_name", "default")
        super(VditorTextField, self).__init__(*args, **kwargs)

    @staticmethod
    def _get_form_class():
        return VditorTextFormField

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self._get_form_class(),
            "config_name": self.config_name,
        }
        defaults.update(kwargs)
        return super(VditorTextField, self).formfield(**defaults)


class VditorTextFormField(forms.fields.CharField):
    def __init__(self, config_name="default", *arg, **kwargs):
        kwargs.update({"widget": VditorWidget(config_name=config_name)})
        super(VditorTextFormField, self).__init__(*arg, **kwargs)
